#!/usr/bin/env python
#coding=utf-8

import onelab
import math, os, sys
import string

OL = onelab.client(__file__)

massFlow  = OL.defineNumber('Foam/Flow/massFlow', value=1.0e-5, label='Mass flow [m3/s]')
viscosity = OL.defineNumber('Foam/Flow/viscosity', value=1.0e-6, label='Viscosity [m2/s]')
nbIter = OL.defineNumber('Foam/Control/nbIter', value=500, label='Nb of iterations')
writeCtrl = OL.defineNumber('Foam/Control/writeCtrl', value=50, label='Writing interval')

# Compute the cross section [mm^2]
diam = OL.defineNumber('Foam/Geometry/diam', value=40,
                       label='Pipe diameter [mm]')
symm = OL.defineNumber('Foam/Geometry/symm', value=1,
                       label='Symmetry (planar/axial)',
                       labels={1:'Rectangular',
                               2:'Tubular'})
width= OL.defineNumber('Foam/Geometry/width', value=1,
                       label='Model thickness [mm]')
angle= OL.defineNumber('Foam/Geometry/angle', value=5,
                       label='Model aperture angle [deg]')

diam  *= 1e-3 # convert to [m]
width *= 1e-3 
angle *= math.pi/180. # convert to [rad]

if symm == 1: #rectangular
    cross = (width/1e-3)*diam/2.
    OL.setNumber('Foam/postpro/cx', visible=0, value=0)
    OL.setNumber('Foam/postpro/cy', visible=0, value=0)
    OL.setNumber('Foam/postpro/cz', visible=0, value=width/2.)
    OL.setNumber('Foam/postpro/nx', visible=0, value=0)
    OL.setNumber('Foam/postpro/ny', visible=0, value=0)
    OL.setNumber('Foam/postpro/nz', visible=0, value=1)
    OL.setNumber('Foam/Geometry/width', visible=1)
    OL.setNumber('Foam/Geometry/angle', visible=0)
if symm == 2: #tubular
    cross = angle/360.*math.pi*diam**2
    OL.setNumber('Foam/postpro/cx', visible=0, value=0)
    OL.setNumber('Foam/postpro/cy', visible=0, value=0)
    OL.setNumber('Foam/postpro/cz', visible=0, value=0)
    OL.setNumber('Foam/postpro/nx', visible=0, value=0)
    OL.setNumber('Foam/postpro/ny', visible=0, value=-math.sin(angle/2.))
    OL.setNumber('Foam/postpro/nz', visible=0, value=math.cos(angle/2.))
    OL.setNumber('Foam/Geometry/width', visible=0)
    OL.setNumber('Foam/Geometry/angle', visible=1)
    
if not OL.loop :
   OL.openProject(OL.checkPath('valve.geo'))
OL.run('mesher', 'gmsh - ' + OL.checkPath('valve.geo'),'-3')

# Reynolds number
lcar = diam/2e3                    #characteristic model length [m]
vcar = massFlow/cross*1e6          # characteristic fluid velocity [m/s]
rey = vcar*lcar/viscosity          # Reynolds number= VL/nu
Reynolds = OL.defineNumber('Foam/Flow/Reynolds',value=rey, readOnly=1, label='Reynolds number')

# preprocess input files
OL.preProcess('foam',OL.checkPath('U.ol'))
OL.preProcess('foam',OL.checkPath('transportProperties.ol'))
OL.preProcess('foam',OL.checkPath('controlDict.ol'))
OL.preProcess('foam',OL.checkPath('sampleDict.ol'))

OL.defineNumber('Foam/Solution/AxialForce', label='Axial force [N]',
	readOnly=1, attributes={'Highlight':'Coral'})
OL.defineNumber('Foam/Solution/DeltaP', label='Pressure jump [Pa]',
	readOnly=1, attributes={'Highlight':'Coral'})

if not OL.loop :
    OL.resetNumberChoices('Foam/Solution/AxialForce')
    OL.resetNumberChoices('Foam/Solution/DeltaP')

if OL.action == 'check' :
   exit(1)

OL.upload(OL.checkPath('U')                  , OL.getPath('foam/0'))
OL.upload(OL.checkPath('transportProperties'), OL.getPath('foam/constant'))
OL.upload(OL.checkPath('controlDict')        , OL.getPath('foam/system'))
OL.upload(OL.checkPath('sampleDict')         , OL.getPath('foam/system'))

OL.call('foam','bash run.sh',rundir=OL.wdir)

frc=onelab.extract('forc.dat',-1,2)
prs=onelab.extract('pres.dat',-1,3)

OL.addNumberChoice('Foam/Solution/AxialForce', frc)
OL.addNumberChoice('Foam/Solution/DeltaP', prs)

OL.download(OL.checkPath('run/U_constantPlane.pos'), OL.getPath())
OL.download(OL.checkPath('run/p_constantPlane.pos'), OL.getPath())

if not OL.batch :
    OL.sendCommand("Delete All;")
    OL.openProject(OL.checkPath('valve.geo'))
    OL.mergeFile(OL.checkPath('p_constantPlane.pos'))
    OL.mergeFile(OL.checkPath('U_constantPlane.pos'))
    OL.sendCommand('BoundingBox{0.13,0.23,-0.06,0.06,0,1};')
else :
    sys.stderr.write('%s: frc = %12.5e (%12.5e)\n' %
                     (__file__, frc, 0.770022))
    sys.stderr.write('%s: prs = %12.5e (%12.5e)\n' %
                     (__file__, prs, 36.6071))
    




